@@ -1,7 +1,9 @@
 /*
   jargrep.c - main functions for jargrep utility
-  Copyright (C) 1999 Bryan Burns
+  Copyright (C) 2002, 2003, 2006 Free Software Foundation
+  Copyright (C) 1999, 2000 Bryan Burns
   Copyright (C) 2000 Cory Hollingsworth 
+  Copyright (C) 2007 Dalibor Topic
  
   Parts of this program are base on Bryan Burns work with fastjar 
   Copyright (C) 1999. 
@@ -18,42 +20,29 @@
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-*/
-
-/* $Id: jargrep.c,v 1.10 2005/08/11 19:38:53 tomcopeland Exp $
-
-$Log: jargrep.c,v $
-Revision 1.10  2005/08/11 19:38:53  tomcopeland
-A tiny fix to get jargrep to compile under GCC 4
-
-Revision 1.9  2000/12/14 23:23:40  toast
-added patches from gcc team
-
-Revision 1.8  2000/09/13 14:02:02  cory
-Reformatted some of the code to more closly match the layout of the orriginal
-fastjar utility.
-
-Revision 1.7  2000/09/12 22:29:36  cory
-Jargrep now seems to do what I want it to do.  Performs properly on Linux x86,
-will test some other platforms later.
-
-
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
 #include <stdio.h>
 #include <unistd.h>
-#include <regex.h>
 #include <errno.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <ctype.h>
-#ifdef STDC_HEADERS
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+
+#include <getopt.h>
+
+#include "regex.h"
+
 #include "jargrep.h"
 #include "jartool.h"
 #include "pushback.h"
@@ -61,9 +50,10 @@ will test some other platforms later.
 #include "zlib.h"
 #include "compress.h"
 
-#define Usage "Usage: %s [-bcinsw] <-e regexp | regexp> file(s)\n"
+static void version(void);
+static void help(const char *name);
 
-extern char *optarg;
+#define Usage "Usage: %s [-bcinsVw] [--version|--help] <-e PATTERN | PATTERN> FILE ...\n"
 
 /*
 Function name: opt_valid
@@ -112,20 +102,20 @@ static regex_t *create_regexp(const char *regstr, int options) {
 				fprintf(stderr, "Error: %s\n", errmsg);
 				free(exp);
 				free(errmsg);
-				exit(1);
+				exit(EXIT_FAILURE);
 			}
 			else {
 				fprintf(stderr, "Malloc of errmsg failed.\n");
 				fprintf(stderr, "Error: %s\n", strerror(errno));
 				free(exp);
-				exit(1);
+				exit(EXIT_FAILURE);
 			}
 		}
 	}
 	else {
 		fprintf(stderr, "Malloc of regex failed,\n");
 		fprintf(stderr, "Error: %s\n", strerror(errno));
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	return exp; 
@@ -257,7 +247,7 @@ static char *read_string(pb_file *pbf, int size) {
 	else {
 		fprintf(stderr, "Malloc of page buffer failed.\n");
 		fprintf(stderr, "Error: %s\n", strerror(errno));
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	return page;
@@ -293,7 +283,7 @@ static char *extract_line(const char *stream, regoff_t begin, regoff_t end, int
 	else {
 		fprintf(stderr, "Malloc failed of output string.\n");
 		fprintf(stderr, "Error: %s\n", strerror(errno));
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	return retstr;
@@ -310,7 +300,6 @@ returns: TRUE if it is a word, FALSE of it is a substring.
 
 static int chk_wrd(regex_t *exp, const char *str) {
 	int wrd_fnd = FALSE;
-	int regflag;
 	int frnt_ok;
 	int bck_ok;
 	const char *str2;
@@ -318,7 +307,7 @@ static int chk_wrd(regex_t *exp, const char *str) {
 
 	str2 = str;
 	frnt_ok = bck_ok = FALSE;
-	while(!wrd_fnd && !(regflag = regexec(exp, str2, 1, &match, 0))) {
+	while(!wrd_fnd && !regexec(exp, str2, 1, &match, 0)) {
 		if(!match.rm_so && (str2 == str)) frnt_ok = TRUE;
 		else if(!isalnum((unsigned char)str2[match.rm_so - 1])
 			&& str2[match.rm_so - 1] != '_')
@@ -397,7 +386,7 @@ purpose:	Verify the CRC matches that as what is stored in the jar file.
 */
 
 static void check_crc(pb_file *pbf, const char *stream, ub4 usize) {
-	ub4 crc;
+	ub4 crc=0;
 	ub4 lcrc;
 	ub1 scratch[16];
 
@@ -405,17 +394,17 @@ static void check_crc(pb_file *pbf, const char *stream, ub4 usize) {
 	crc = crc32(crc, (const unsigned char *)stream, usize);
 	if(pb_read(pbf, scratch, 16) != 16) {
 		perror("read");
-        exit(1);
+        exit(EXIT_FAILURE);
 	}
 	if(UNPACK_UB4(scratch, 0) != 0x08074b50) {
 		fprintf(stderr, "Error! Missing data descriptor!\n");
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 	lcrc = UNPACK_UB4(scratch, 4);
 	if(crc != lcrc){
     	fprintf(stderr, "Error! CRCs do not match! Got %x, expected %x\n",
               crc, lcrc);
-      	exit(1);
+      	exit(EXIT_FAILURE);
     }
 }
 
@@ -472,7 +461,7 @@ static regmatch_t *fnd_match(regex_t *exp, const char *str_stream, int *i) {
 		else {
 			fprintf(stderr, "Realloc of match_array failed.\n");
 			fprintf(stderr, "Error: %s\n", strerror(errno));
-			exit(1);
+			exit(EXIT_FAILURE);
 		}
 	} 
 
@@ -507,7 +496,7 @@ static int cont_grep(regex_t *exp, regex_t *nl_exp, int fd, pb_file *pbf, int op
 	char *filename;
 	char *str_stream;
 	regmatch_t *match_array;
-	regmatch_t *nl_offsets;
+	regmatch_t *nl_offsets=0;
 
 	if(pb_read(pbf, (file_header + 4), 26) != 26) {
 		perror("read");
@@ -575,13 +564,26 @@ static void jargrep(regex_t *exp, regex_t *nl_exp, const char *jarfile, int opti
 					floop = FALSE;
 					break;
 				case 2:
-          continue; /* fall through continue */
+					/* fall through continue */
+					;
 				}
 			}
 		} while(floop);
 	}
 }
 
+/* This is used to mark options with no short value.  */
+#define LONG_OPT(Num)  ((Num) + 128)
+
+#define OPT_HELP     LONG_OPT (0)
+
+static const struct option option_vec[] =
+{
+  { "help", no_argument, NULL, OPT_HELP },
+  { "version", no_argument, NULL, 'V' },
+  { NULL, no_argument, NULL, 0 }
+};
+
 /*
 Funtion Name: main
 args:	argc	number of in coming args.
@@ -600,7 +602,8 @@ int main(int argc, char **argv) {
 	regex_t *nl_exp = NULL;
 	char *regexpstr = NULL;
 
-	while((c = getopt(argc, argv, "bce:insVw")) != -1) {
+	while((c = getopt_long(argc, argv, "bce:insVw",
+			       option_vec, NULL)) != -1) {
 		switch(c) {
 			case 'b':
 				options |= JG_PRINT_BYTEOFFSET;
@@ -612,7 +615,7 @@ int main(int argc, char **argv) {
 				if(!(regexpstr = (char *) malloc(strlen(optarg) + 1))) {
 					fprintf(stderr, "Malloc failure.\n");
 					fprintf(stderr, "Error: %s\n", strerror(errno));
-					exit(1);
+					exit(EXIT_FAILURE);
 				}
 				strcpy(regexpstr, optarg);
 				break;
@@ -629,15 +632,17 @@ int main(int argc, char **argv) {
 				options |= JG_INVERT;
 				break;
 			case 'V':
-				printf("%s\n", GVERSION);
-				exit(0);
+				version ();
+				break;
 			case 'w':
 				options |= JG_WORD_EXPRESSIONS;
 				break;
+			case OPT_HELP:
+				help(argv[0]);
+				break;
 			default:
-				fprintf(stderr, "Unknown option -%c\n", c);
 				fprintf(stderr, Usage, argv[0]);
-				exit(1);
+				exit(EXIT_FAILURE);
 		}
 	}
 	if(!regexpstr){
@@ -648,7 +653,7 @@ int main(int argc, char **argv) {
 		else {
 			fprintf(stderr, "Invalid arguments.\n");
 			fprintf(stderr, Usage, argv[0]);
-			exit(1);
+			exit(EXIT_FAILURE);
 		}
 	}
 	else if((argc - optind) == 1) {
@@ -657,7 +662,7 @@ int main(int argc, char **argv) {
 	else {
 		fprintf(stderr, "Invalid arguments.\n");
 		fprintf(stderr, Usage, argv[0]);
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	if(opt_valid(options)) {
@@ -676,3 +681,36 @@ int main(int argc, char **argv) {
 
 	return retval;
 }
+
+void help(const char *filename)
+{
+  printf (Usage, filename);
+  printf ("\
+\n\
+Search files in a jar file for a pattern.\n\
+\n\
+   -b                print byte offset of match\n\
+   -c                print number of matches\n\
+   -i                compare case-insensitively\n\
+   -n                print line number of each match\n\
+   -s                suppress error messages\n\
+   -w                force PATTERN to match only whole words\n\
+   -e PATTERN        use PATTERN as regular expression\n\
+   -V|--version      print version number and exit\n\
+   --help            print help\n\
+");
+
+  exit(EXIT_SUCCESS);
+}
+
+void version (void)
+{
+  printf("grepjar (%s) %s\n\n", PACKAGE, VERSION);
+  printf("Copyright 1999, 2000, 2001  Bryan Burns\n");
+  printf("Copyright 2000 Cory Hollingsworth\n");
+  printf("Copyright 2006 Free Software Foundation\n");
+  printf("\
+This is free software; see the source for copying conditions.  There is NO\n\
+warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n");
+  exit(EXIT_SUCCESS);
+}
